#include <iostream>
#include <vector>
#include <queue>

#define endl '\n'

using namespace std;

const int INF = 1e9 + 3;

const int MAXN = 2e5 + 3;

const int MAX_AIRPLANES = 2;
const int BUS = 0;
const int AIRPLANE = 1;

vector<pair<int, int>> graph[MAXN];

int dist[MAXN][MAX_AIRPLANES + 1];
bool inQueue[MAXN][MAX_AIRPLANES + 1];

int find_min_dist(int n, int maxAirplanes) {
    int startCity = 0;
    int endCity = n - 1;

    for (int i = 0; i < n; ++i) {
        for (int j = 0; j <= MAX_AIRPLANES; ++j) {
            dist[i][j] = INF;
            inQueue[i][j] = false;
        }
    }

    queue<pair<int, int>> spfa;

    spfa.push({startCity, 0});
    inQueue[startCity][0] = true;

    dist[startCity][0] = 0;

    while (!spfa.empty()) {
        auto [currentCity, usedAirplanes] = spfa.front();

        spfa.pop();
        inQueue[currentCity][usedAirplanes] = false;

        for (auto [nextCity, vehicleType]: graph[currentCity]) {
            int neededAirplanes = usedAirplanes + (vehicleType == AIRPLANE);

            if (neededAirplanes > maxAirplanes) {
                continue;
            }

            int newDistance = dist[currentCity][usedAirplanes] + 1;

            if (newDistance >= dist[nextCity][neededAirplanes]) {
                continue;
            }

            dist[nextCity][neededAirplanes] = newDistance;

            if (!inQueue[nextCity][neededAirplanes]) {
                spfa.push({nextCity, neededAirplanes});
                inQueue[nextCity][neededAirplanes] = true;
            }
        }
    }

    int result = INF;
    for (int i = 0; i <= maxAirplanes; ++i) {
        result = min(result, dist[endCity][i]);
    }

    return (result == INF ? -1 : result);
}

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, m, k;
    cin >> n >> m >> k;

    for (int i = 0; i < m; ++i) {
        int from, to, type;
        cin >> from >> to >> type;

        --from;
        --to;
        --type;

        graph[from].push_back({to, type});
        graph[to].push_back({from, type});
    }

    cout << find_min_dist(n, k) << endl;

    return 0;
}
