#include <iostream>
#include <vector>
#include <algorithm>
#include <math.h>

#include "segment.h"

using namespace std;

const int INF = 2e9 + 3;

const int MAXN = 3e6 + 3;
const int MAX_LOG = 23;

int rmq[MAXN][MAX_LOG];
int logTwo[MAXN];

void calc_rmq(int n, vector<int> &nums) {
    for (int i = 2; i < MAXN; ++i) {
        logTwo[i] = logTwo[i / 2] + 1;
    }

    for (int i = 0; i < n; ++i) {
        rmq[i][0] = nums[i];
    }

    for (int interval = 1; interval < MAX_LOG; ++interval) {
        for (int i = 0; i + (1 << interval) - 1 < n; ++i) {
            int j = i + (1 << (interval - 1));

            rmq[i][interval] = min(rmq[i][interval - 1], rmq[j][interval - 1]);
        }
    }
}

int query_min(int l, int r) {
    if (l > r) {
        return INF;
    }

    int interval = logTwo[r - l + 1];

    return min(rmq[l][interval], rmq[r - (1 << interval) + 1][interval]);
}

int tree[4 * MAXN];

void build_tree(int ind, int tl, int tr, vector<int> &nums) {
    if (tl == tr) {
        tree[ind] = nums[tl];
        return;
    }

    int mid = (tl + tr) / 2;

    build_tree(2 * ind, tl, mid, nums);
    build_tree(2 * ind + 1, mid + 1, tr, nums);

    tree[ind] = __gcd(tree[2 * ind], tree[2 * ind + 1]);
}

int query_gcd(int ind, int tl, int tr, int targetL, int targetR) {
    if (tl > tr or targetL > tr or targetR < tl) {
        return 0;
    }

    if (targetL <= tl and tr <= targetR) {
        return tree[ind];
    }

    int mid = (tl + tr) / 2;

    int result = 0;

    if (targetL <= mid) {
        result = __gcd(result, query_gcd(2 * ind, tl, mid, targetL, targetR));
    }

    if (targetR >= mid + 1) {
        result = __gcd(result, query_gcd(2 * ind + 1, mid + 1, tr, targetL, targetR));
    }

    return result;
}

long long max_segment(vector<int> nums) {
    int n = (int) nums.size();

    build_tree(1, 0, n - 1, nums);

    calc_rmq(n, nums);

    long long result = n;

    int minNum = INF;

    for (int i = 0; i < n; ++i) {
        int l = 0, r = i - 1;

        while (l <= r) {
            int mid = (l + r) / 2;

            if (query_min(mid, i) >= nums[i]) {
                r = mid - 1;
            } else {
                l = mid + 1;
            }
        }

        int leftTarget = r + 1;

        l = i + 1;
        r = n - 1;

        while (l <= r) {
            int mid = (l + r) / 2;

            if (query_min(i, mid) >= nums[i]) {
                l = mid + 1;
            } else {
                r = mid - 1;
            }
        }

        int rightTarget = l - 1;

        // cout << "(" << i << ", " << nums[i] << ") --> " << leftTarget << " " << rightTarget << endl;

        result = max(result, 1ll * (rightTarget - leftTarget + 1) * nums[i]);
    }

    return result;
}

