#include <iostream>
#include <vector>
#include <algorithm>

#define endl '\n'

using namespace std;

const int NO_COLOR = -1;

void solve() {
    int n, m, d, k;
    cin >> n >> m >> d >> k;

    int sameColorLimit = d / k;

    vector<vector<int>> graph(n);
    vector<int> inDegree(n);

    for (int i = 0; i < m; ++i) {
        int from, to;
        cin >> from >> to;

        --from;
        --to;

        graph[from].push_back(to);
        graph[to].push_back(from);

        ++inDegree[from];
        ++inDegree[to];
    }

    vector<pair<int, int>> allNodeDegrees(n);

    for (int i = 0; i < n; ++i) {
        allNodeDegrees[i] = {inDegree[i], i};
    }

    sort(allNodeDegrees.begin(), allNodeDegrees.end());

    vector<int> colors(n, NO_COLOR);
    int currentColor = 0;

    for (int i = 0; i < n; ++i) {
        colors[i] = currentColor;

        ++currentColor;

        if (currentColor >= k) {
            currentColor = 0;
        }
    }

    vector<bool> visited(n, false);

    for (int i = n - 1; i >= 0; --i) {
        int currentNode = allNodeDegrees[i].second;

        vector<int> filteredNodes;
        int currentSameColorCounter = 0;

        for (int nextNode : graph[currentNode]) {
            if (colors[currentNode] != colors[nextNode]) {
                continue;
            }

            ++currentSameColorCounter;

            if (!visited[nextNode]) {
                filteredNodes.push_back(nextNode);
            }
        }

        // Dummy node
        filteredNodes.push_back(n);

        sort(filteredNodes.begin(), filteredNodes.end());

        int sameEdges = 1;
        int newColor = (colors[currentNode] == k - 1) ? 0 : colors[currentNode];

        for (int j = 1; j < (int) filteredNodes.size(); ++j) {
            if (filteredNodes[j] == filteredNodes[j - 1]) {
                ++sameEdges;
                continue;
            }

            int targetNode = filteredNodes[j - 1];

            if (sameEdges > 1) {
                currentSameColorCounter -= sameEdges;

                colors[targetNode] = newColor;
                ++newColor;

                if (newColor >= k) {
                    newColor = 0;
                }
            }

            sameEdges = 1;
        }

        for (int j = 1; j < (int) filteredNodes.size(); ++j) {
            if (filteredNodes[j] == filteredNodes[j - 1]) {
                ++sameEdges;
                continue;
            }

            int targetNode = filteredNodes[j - 1];

            if (sameEdges == 1) {
                --currentSameColorCounter;

                colors[targetNode] = newColor;
                ++newColor;

                if (newColor >= k) {
                    newColor = 0;
                }
            }

            sameEdges = 1;
        }

        visited[currentNode] = true;
    }

    for (int i = 0; i < n; ++i) {
        cout << colors[i] + 1 << " ";
    }
    cout << endl;
}

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int tests;
    cin >> tests;;

    for (int test = 0; test < tests; ++test) {
        solve();
    }

    return 0;
}
